%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Task Force on Modern Heuristic Optimization Test Beds
% Working Group on Modern Heuristic Optimization
% Intelligent Systems Subcommittee
% Power System Analysis, Computing, and Economic Committee
%
% 2018 Grid Optimization Competition
% Evaluating the Performance of Modern Heuristic Optimizers on Stochastic Optimization Problems applied to Smart Grids
%
% Test bed A:
% Stochastic OPF in Presence of Renewable Energy and Controllable Loads
%
% Based on 2014 and 2017 competition AND [1], [2]
%
% [1] J. Arvalo, F. Santos, S. Rivera, Uncertainty cost functions for solar photovoltaic generation, wind
% energy generation, and plug-in electric vehicles: Mathematical expected value and verification by monte
% carlo simulation, International Journal of Power and Energy Conversion, in press, 2018.
% http://www.inderscience.com/info/ingeneral/forthcoming.php?jcode=ijpec
% [2] S. Rivera, A. Romero, Dispatch Modeling Incorporating Maneuver Components, Wind Power and
% Electric Vehicles Penetration, using Heuristic Techniques, Application of Modern Heuristic Optimization
% Techniques in Power and Energy Systems, ed: Wiley, in press, 2018.
%
% Questions can be directed to Sergio Rivera srriverar@unal.edu.co 
%
% This routine is called subsequent to every function evaluation, 
% i.e. power flow calculation. It does not affect the calculations 
% done in test_bed_OPF.p, which calculates internally the fitness by
% using static penalty constraint handling method.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [f,g]=constraint_handling(o,g)
    % You may use procedural information 
    % such as function evaluation counter 
    % proc.i_eval to dynamically adjust
    % your constraint handling method.
    global ps
    
    % Uniform penalty coefficient.
	penalty=1e7;
    % Sum up all violations.
    % Note that, at this point, constraint 
    % vector g consists only of elements
    % >=0 due to g(g<1e-4)=0 as internal
    % preliminary step.
    
    [ ~, y_size ] = size( g );
    
    g_slack = g( 1 );
    g_v = sum( g( 2 : 1 + 2 * ps.n_load ) );
    g_MVA_MVAR = sum( g( 1 + 2 * ps.n_load + 1 : y_size ) );
    
    g = [ g_slack/100 g_v g_MVA_MVAR/100 ];
  	g=sum(g.*g);
    
    % Fitness function.
	f=o+penalty*g;
end