%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Task Force on Modern Heuristic Optimization Test Beds
% Working Group on Modern Heuristic Optimization
% Intelligent Systems Subcommittee
% Power System Analysis, Computing, and Economic Committee
%
% 2018 Grid Optimization Competition
% Evaluating the Performance of Modern Heuristic Optimizers on Stochastic Optimization Problems applied to Smart Grids
%
% Test bed A:
% Stochastic OPF in Presence of Renewable Energy and Controllable Loads
%
% Based on 2014 and 2017 competition AND [1], [2]
%
% [1] J. Arvalo, F. Santos, S. Rivera, Uncertainty cost functions for solar photovoltaic generation, wind
% energy generation, and plug-in electric vehicles: Mathematical expected value and verification by monte
% carlo simulation, International Journal of Power and Energy Conversion, in press, 2018.
% http://www.inderscience.com/info/ingeneral/forthcoming.php?jcode=ijpec
% [2] S. Rivera, A. Romero, Dispatch Modeling Incorporating Maneuver Components, Wind Power and
% Electric Vehicles Penetration, using Heuristic Techniques, Application of Modern Heuristic Optimization
% Techniques in Power and Energy Systems, ed: Wiley, in press, 2018.
%
% Questions can be directed to Sergio Rivera srriverar@unal.edu.co
% 
% Incorporating basic Particle Swarm Optimization (PSO) algorithm as 
% optimization engine to solve test bed declarations.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function psopt(fhd,ii,jj,kk,args)
    % Paste and adapt the following 
    % parts to your implementation.
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	global proc
    global ps
    % Reinitialization of local
    % function evaluation counter.
    proc.i_eval=0;
    % Function evaluation at which
    % the last update of the global
    % best solution occurred.
    % Refers to the internal evaluation
    % using static penalty constraint 
    % handling method.
	proc.last_improvement=1;
    % Signalizing your 
    % to stop running.
    proc.finish=0;   
    % Size of the swarm.
    n_par=proc.pop_size;
    % For dynamic weight adaption.
    me=proc.n_eval;
    % Dimensionality of test case.
	
    D=ps.D+4; % PLEASE NOTE THAT 4 CONTROLABLE LOADS WERE ADDED
    % Individuals' lower bounds.
    VRmin=[ps.x_min  50 300  0  0]; % PLEASE NOTE THAT 4 LOWER BOUNDS OF CONTROLABLE LOADS WERE ADDED REGARDING 2017 TEST BED 1
    % Individuals' upper bounds.
    VRmax=[ps.x_max 158 420 53 53]; % PLEASE NOTE THAT 4 UPPER BOUNDS OF CONTROLABLE LOADS WERE ADDED REGARDING 2017 TEST BED 1
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    cc=[2 2];
    iwt=0.9-(1:me).*(0.5./me);
    if length(VRmin)==1
        VRmin=repmat(VRmin,1,D);
        VRmax=repmat(VRmax,1,D);
    end
    VRmin=repmat(VRmin,n_par,1);
    VRmax=repmat(VRmax,n_par,1);
    Vmin=VRmin;
    Vmax=VRmax;
    pos=VRmin+(VRmax-VRmin).*rand(n_par,D);
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	[fit,obj,g_sum,pos,fit_best]=feval(fhd,ii,jj,kk,args,pos);
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    vel=Vmin+2.*Vmax.*rand(n_par,D);
    pbest=pos;
    pbestval=fit;
    [gbestval,gbestid]=min(pbestval);
    gbest=pbest(gbestid,:);
    gbestrep=repmat(gbest,n_par,1);

    i=2;
    
    % Instead of your common stopping
    % criterion, use an undefined 
    % expression at this point.
    % Your implementation will be
    % cancelled intrinsically by flag
    % proc.finish from test bed
    % declarations in test_bed_OPF.p.
    %%%%%%%
    while 1
    %%%%%%%
        aa=cc(1).*rand(n_par,D).*(pbest-pos)+cc(2)...
            *rand(n_par,D).*(gbestrep-pos);
        vel=iwt(i).*vel+aa;
        vel=(vel>Vmax).*Vmax+(vel<=Vmax).*vel;
        vel=(vel<Vmin).*Vmin+(vel>=Vmin).*vel;
        pos=pos+vel;
        pos=((pos>=VRmin)&(pos<=VRmax)).*pos+...
            (pos<VRmin).*(VRmin+0.25.*(VRmax-VRmin).*rand(n_par,D))+...
            (pos>VRmax).*(VRmax-0.25.*(VRmax-VRmin).*rand(n_par,D));
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        [fit,obj,g_sum,pos,fit_best]=feval(fhd,ii,jj,kk,args,pos);
        %fit_best
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
        tmp=(pbestval<fit);
        temp=repmat(tmp',1,D);
        pbest=temp.*pbest+(1-temp).*pos;
        pbestval=tmp.*pbestval+(1-tmp).*fit;
        [gbestval,tmp]=min(pbestval);
        gbest=pbest(tmp,:);
        gbestrep=repmat(gbest,n_par,1);
        
        %%%%%%%%%%%%%%
        if proc.finish
            return;
        end
        %%%%%%%%%%%%%%
        
        % To prematurely stop
        % running the current trial,
        % you may use a statement
        % as the following.
        % However, note that
        % storage of intermediate 
        % results must be done 
        % manually since no ASCII
        % file output will be 
        % provided in that case.
        % As an example, retrieve
        % relevant information
        % from cell array res.
%         %%%%%%%%%%%%%%%%%%%%
%         if proc.i_eval>=1000
%             return;
%         end
%         %%%%%%%%%%%%%%%%%%%%
    end
end